package NTLSystem.features;

import com.sun.java.swing.*;
import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.Vector;

/**
 * A string feature-descriptor describes a feature that can have any string value,
   and contains meta information that pertains to all instantiations of
   a particular string-based feature.
   Note that features of this type can only be determinate.
   <P>
   $Id: StringFeatureDescriptor.java,v 1.12 1998/10/16 01:57:15 jsled Exp $
   <BR>Last editor: $Author: jsled $
 
   @author     Tajh Taylor [tajh@icsi.berkeley.edu]
   @version    $Revision: 1.12 $
 **/

public class StringFeatureDescriptor
extends FeatureDescriptor
{
  /**
   * serialization Stream Unique Identifier.  Recompute [with
   * serialver] when this class changes.
   **/
  static final long serialVersionUID = 9138189220514526586L;

   // private instance variables

   // public instance methods

   /** A no-arg constructor for Serialization. */
   public StringFeatureDescriptor()
   {
     super();
     setProbabilistic(false);
     setDefaultValue( new String("") );
   }

   /** A constructor for determinate feature descriptors.
       @param newName         the new name for this feature descriptor.
       @param newDefaultValue the new default value for this feature descriptor */
   public StringFeatureDescriptor(String newName, String newDefaultValue)
   {
      super(newName);
      setProbabilistic(false);
      setDefaultValue(newDefaultValue);
   }



   // meta-info reporting methods

   /** Report the default value.
       @return a string containing the default value */
  
  /**
   * Sets the default value.
   *
   * @param     newValue The new default value to use.
   **/
  protected void setDefaultValue(String newValue)
  {
    defaultValue = newValue;
  } 

   /** Tell whether or not this descriptor is for a
       feature with ordered values, always false.
       @return boolean flag indicating orderedness, always false. */
   public boolean isOrdered()
   {
      return false;
   }

  /**
   * @param     o An Object to be tested to see if it is a valid value
   *            of this particular feature.
   * @return    True if the object is valid, false otherwise.
   **/
  public boolean isValidValue(Object o)
  {
    return (o instanceof String);
  }

  /**
   * @see FeatureDescriptor
   **/
  public Object valueFromText(String textVal)
       throws IllegalArgumentException
  {
    return textVal;
  }

  /**
   * @see FeatureDescriptor
   **/
  public String valueAsText(Object val)
  {
    return (String)val;
  }


   // interface implementations

   /** Read in a descriptor written in human-readable text form.
       @param input An ObjectInput object from which the text will
                    be read. */
   public void  readExternal(ObjectInput input)
       // throws IOException
   {
      // read the header
      //TODO readExternal() method needs a generic word parser class
   }

   /** Write the descriptor out in human-readable text form.
       @param output An ObjectOutput object to which the text will
                     be written. */
   public void  writeExternal(ObjectOutput output)
       //throws IOException
   {
   }

  public String getDisplayableName()
  {
    return new String("String Feature Descriptor");
  }

  protected JPanel getEditViewPanel(boolean edit)
  { 
    viewer = new SFDViewer(this, edit);
    return viewer;
  }

}
 
/* INNER CLASSES FOR EDITING */
class SFDViewer
    extends FDViewer
{
    /**
     * A JTextField for editing the default value.
     **/
    private JTextField defValueTF;

    public SFDViewer(FeatureDescriptor this_fd, boolean edit)
    {
	super(this_fd, edit);
	// Make the probabilistic radiobutton disabled, as StringFDs can
	// only be Determinisitic.
	prob.setEnabled(false);
    }

    public void createProbPanel()
    {
	; // noop ... StringFeatureDescriptors only have a deterministic
	// value.
    }

    public void createDetPanel()
    {
	String defVal = (String)fd.getDefaultValue();
	if (defVal == null) {
	    defVal = new String("");
	}
	defValueTF = new JTextField( defVal, 25 );
	if (viewing) {
	    defValueTF.setEditable(false);
	}
	detPanel = new JPanel();
	// Add the Label
	detPanel.add(new JLabel("Default Value:"));
	// Add the text field.
	detPanel.add(defValueTF);
    }

    public String getFDTypeName()
    {
	return new String("String Feature Descriptor");
    }

    public void saveValues()
	throws CannotSaveValuesException
    {
	super.saveValues();
	((StringFeatureDescriptor)fd).setDefaultValue( defValueTF.getText() );
    }

    public void canSaveValues(Vector reasons)
    {
	// It's always possible, since we allow "" as a default value.
	super.canSaveValues(reasons);
    }

}
