package NTLSystem.support;

import java.beans.*;
import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A class for testing the PropertyCollectionSheet editing
 * capabilities and all the stuff that goes along with it.
 * <P> 
 * $Id: PropCollSheetTest.java,v 1.6 1998/06/20 01:08:25 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author Josh Sled [jsled@icsi.berkeley.edu]
 * @see PropertyCollection
 * @see PropertyCollectionSheet
 * @version $Id: PropCollSheetTest.java,v 1.6 1998/06/20 01:08:25 jsled Exp $<BR>
 **/
public class PropCollSheetTest
implements PropertyCollection
{

  /* ATTRIBUTES */

  /** The data being stored. */
  private Object data[];

  /** The names of the data. */
  private String names[];

  /** A name for this Fstruct. */
  private String name;

  /* CONSTRUCTORS */

  /** The constructor. **/
  public PropCollSheetTest(Object data[], String names[], String name)
  {
    this.data = data;
    this.names = names;
    this.name = name;
  }



  /* METHODS */

  public String getLabel()
  {
    return new String(name);
  }


  public String getName()
  {
    return getLabel();
  }

  public void setName(String name)
  {
    this.name = name;
  }


  public EditablePropertyDescriptor[] getEditableProperties()
  {
    int numProps = data.length + 1;
    EditablePropertyDescriptor toRet[] = new EditablePropertyDescriptor[numProps];

    Method getter = getGetter();
    Method setter = getSetter();
    Method getName = null;
    Method setName = null;
    try {
      Class getParms[] = { };
      Class setParms[] = { Class.forName("java.lang.String") };
      getName = this.getClass().getMethod("getName", getParms);
      setName = this.getClass().getMethod("setName", setParms);
    } catch (ClassNotFoundException cnfe) {
      // This shouldn't occur.
    } catch (NoSuchMethodException nsme) {
      // This shouldn't occur.
    } // end try-catch

    for (int i=0; i<data.length; i++) {
      toRet[i] = new EditablePropertyDescriptor(new String(names[i]), data[i].getClass(), getter, setter, new Integer(i));
    } // end for
    
    // Set the last element of the array [the name field].
    toRet[numProps-1] = new EditablePropertyDescriptor(new String("Name"), name.getClass(), getName, setName, null);
    return toRet;
  }



  protected Method getGetter()
  {
    Method toRet = null;
    try {
      Class parms[] = { Class.forName("java.lang.Integer") };
      toRet = this.getClass().getMethod("GetSlot", parms);
    } catch (ClassNotFoundException cnfe) {
      // This shouldn't occur.
    } catch (NoSuchMethodException nsme) {
      // This shouldn't occur.
    } // end try-catch
    return toRet;
  }



  protected Method getSetter()
  {
    Method toRet = null;
    try {
      Class parms[] = { Class.forName("java.lang.Integer"),
			Class.forName("java.lang.Object") };
      toRet = this.getClass().getMethod("SetSlot", parms);
    } catch (Exception e) {
      // Neither ClassNotFoundException nor NoSuchMethodException
      // should occur. 
    } // end try-catch
    return toRet;
  }



  /**
   * Edits the PropCollSheetTest.  This is the fun stuff, because this is where
   * we play with the JavaBeans code.
   **/
  public void Edit()
  {
    // Create a new PropertyCollectionSheet to edit this PropCollSheetTest
    Frame pcs;
    WindowWaiter ww = new WindowWaiter();
    pcs = new PropertyCollectionSheet(this, 200, 500);
    pcs.addWindowListener(ww);
    ww.WaitUntilDone();
    
  }



  /**
   * Returns a multi-line string of the PropCollSheetTest for checking.
   * Simply calls toString() on the objects in the data array.
   **/
  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    sb.append("PropCollSheetTest named: ");
    sb.append(getName() + "\n");
    for (int i=0; i<data.length; i++) {
      sb.append(names[i] + " <" +
		data[i].getClass().toString() + ">: " +
		data[i].toString() + "\n");
    }

    return sb.toString();
  }



  /**
   * The Get accessor method for an PropCollSheetTest.
   *
   * @param     i The index of the object to get.
   **/ 
  public Object GetSlot(Integer i)
  {
    return data[i.intValue()];
  }



  /**
   * The Set accessor method for an PropCollSheetTest.
   *
   * @param     i The index of the object to get.
   * @param     o The object to set that slot to.
   * @returns   The old value of that slot.
   **/
  public Object SetSlot(Integer i, Object o)
  {
    Object toRet = data[i.intValue()];
    data[i.intValue()] = o;
    return toRet;
  }



  /** The Main code for testing. **/
  public static void main(String args[])
  {

    MultivaluedAttribute ma;
    PropCollSheetTest pcst;

    // Create some names for the objects below.
    String names[] = {
      new String("Boolean"),
      new String("Color"),
      new String("Font"),
      new String("Double"),
      new String("Integer"),
      new String("Long"),
      new String("String0"),
      new String("String1"),
      new String("String2"),
      new String("MultivaluedAttribute")
    };
    
    ma = new MultivaluedAttribute(names);
    ma.setValue("Long");
    
    // Create some objects to store in the PropCollSheetTest.
    Object objs[] = {
      new Boolean(true),
      new Color(Color.blue.getRGB()),
      new Font("Helvetica", Font.PLAIN, 32),
      new Double(3.1459),
      new Integer(42),
      new Long(42000000000000L),
      new String("Test - 42 - 3.1459"),
      new String("Josh Sled Rulez d00d!"),
      new String("Get Bent"),
      ma
    };
    
    // Create an PropCollSheetTest out of those objects.
    pcst = new PropCollSheetTest(objs, names, "Test PropertyCollection");

    // Print it out as a string.
    System.out.println(pcst.toString());

    // Set the Integer editor to "IntEditor" and the Boolean editor to "BoolEditor"
    try {
      PropertyEditorManager.registerEditor(Class.forName("java.lang.Integer"), Class.forName("sun.beans.editors.IntEditor"));
      PropertyEditorManager.registerEditor(Class.forName("java.lang.Boolean"), Class.forName("sun.beans.editors.BoolEditor"));
    } catch (ClassNotFoundException cnfe) {
      System.out.println("Could not register editors");
      System.out.println(cnfe.getMessage());
      cnfe.printStackTrace();
    } // end try-catch

    // Edit it.
    pcst.Edit();

    // Print it out again to make sure that the editing works.
    System.out.println(pcst.toString());

    // Exit
    System.exit(0);
  }
}

