package NTLSystem.support;

import com.sun.java.swing.AbstractListModel;
import com.sun.java.swing.ComboBoxModel;
import java.util.Vector;

/**
 * Creates a ComboBoxModel for a Vector of Strings.  Unfortunately, one of
 * the cool things about ListModel is lost in this arangement, which
 * is automagic updating of the List when the Vector changes.  With
 * this class, the 'update' method will need to be called after Vector
 * changes.
 * <P>
 * The other thing of note is that the Vector <EM>must</EM> contain
 * Strings [or, the 'toString()' method of the objects in the Vector
 * must only return their name, and nothing else].
 * <P>
 * $Id: VectorComboBoxModel.java,v 1.4 1998/09/03 02:47:17 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author    Joshua Sled [jsled@icsi.berkeley.edu]
 * @see       com.sun.java.swing.ListModel
 * @see       com.sun.java.swing.AbstractListModel
 * @see       com.sun.java.swing.ComboBoxModel
 * @version   $Revision: 1.4 $
 **/
public class VectorComboBoxModel
    extends AbstractListModel
    implements ComboBoxModel
{
    // {{{ Attributes

    /**
     * The Vector we're modeling.
     **/
    protected Vector vec;

    /**
     * The currently-selected Object.
     **/
    protected Object sel;

    // }}} Attributes

    // {{{ Constructors

    /**
     * Constructs a ListModel for the given Vector.
     *
     * @param     v  The Vector to model.
     **/
    public VectorComboBoxModel(Vector v)
    {
	super();
	vec = v;
	if (vec.size() > 0) {
	    sel = getElementAt(0);
	} else {
	    sel = null;
	}
    }

    // }}} Constructors

    // {{{ Methods

    public int getSize()
    {
	return vec.size();
    }

    public Object getElementAt(int i)
    {
	return vec.elementAt(i);
    }

    public Object getSelectedItem()
    { 
	return sel;
    }

    public void setSelectedItem(Object o)
    {
	sel = o;
	fireContentsChanged( this, 0, 0 );
    }

    /**
     * Updates the contents of this VectorListModel.  Basically, just
     * fires a ContentsChanged ListDataEvent on the ranges of the
     * Vector.
     **/
    public void update()
    {
	fireContentsChanged( this, 0, vec.size() );
    }

    // }}} Methods

}
