
///////////////////////////////////////////////////////////////////////
// Linking Feature Structures

class Fstruct implements Serializable {

  private Hashtable feats;  // maps String feature names to String values
*** features

  private Hashtable featsOff;  // maps String feature names to String values
*** NONE (features, property ignored?)

*** done
  // Construct an empty Fstruct.
  Fstruct() {
    //    type = new String();
    feats = new Hashtable();
    featsOff = new Hashtable();
  }

*** replace  by cloning
  // Construct an Fstruct by copying another one.
  Fstruct(Fstruct f) {
    this();
    //    type = copyValueOf(f.type); // NC
    Enumeration featEnum = f.features();
    while (featEnum.hasMoreElements()) {
      String feat = (String)featEnum.nextElement(); 
      set(feat, f.get(feat)); 
    }
    featEnum = f.featuresOff();
    while (featEnum.hasMoreElements()) {
      String feat = (String)featEnum.nextElement();
      set(feat, f.get(feat));
    }
  }
  
*** replace by ?? 
  // Construct from a string of the form "f1=v1 f2=v2 f3=v3".
  Fstruct(String s) {
    this();
    if (s == null || s.trim().equals("")) return;
    try {
      NestedStreamTokenizer input = new 
	NestedStreamTokenizer(new StringReader(s));
      int token = input.nextToken();
      while (token != NestedStreamTokenizer.TT_EOF) {
	if (token != NestedStreamTokenizer.TT_WORD) {
	  throw(new IOException("expected feature name"));
	}
	String f = input.sval;
	token = input.nextToken();
	if (token != '=') {
	  throw(new IOException("expected '='"));
	}
	token = input.nextToken();
	if (token != NestedStreamTokenizer.TT_WORD) {
	  throw(new IOException("expected feature value"));
	}
	set(f, input.sval);
	token = input.nextToken();
      }
    }
    catch (IOException e) {
      System.err.println("Fstruct-from-string exception: " + e.getMessage());
      feats = new Hashtable();
    }
  }
  

*** 
  // Construct a Fstruct corresponding to the MAP feature values in a
  // PFstruct (but only for those features whose peakedness >= minSetFeature).
  Fstruct(PFstruct pfstruct) { this(singletonArray(pfstruct)); }

  // Helper to the constructor Fstruct(PFstruct):
  private static PFstruct[] singletonArray(PFstruct pf) {
    PFstruct[] result = new PFstruct[1];
    result[0] = pf;
    return result;
  }

  // Construct a Fstruct from an array of PFstructs, resolving
  // conflicts on a feature-by-feature basis by choosing the MAP value
  // from the most peaked distribution across the PFstructs for each
  // feature, so long as the peakedness >= minSetFeature.  Entries
  // in the array may be null.
  Fstruct(PFstruct[] pfstructs) {
    this();
    // Loop through all linking features:
    Enumeration fgen = VerbLearn.context().params().featureNames();
    while (fgen.hasMoreElements()) {
      String f = (String)fgen.nextElement();
      // Choose value from peakiest distribution:
      int winner = -1;
      double winnerPeakedness = -1;
      for (int i = 0; i < pfstructs.length; i++) {
	if (pfstructs[i] != null) {
	  // nc added! is it right????
	  if (pfstructs[i].getDist(f) != null) {
	    double currentPeakedness = pfstructs[i].getDist(f).peakedness();
	    if (currentPeakedness > winnerPeakedness) {
	      winner = i; // slot #?
	      winnerPeakedness = currentPeakedness;
	    }
	  }
	  else {
	    // nothing right now... 
	  }
	}
      }
      // Only set the feature if the winner's peakedness >= minSetFeature:
      // BUT IF THE FEATURE NAME IS "SCHEMA", SET IT REGARDLESS OF HOW
      // LOW THE PEAKEDNESS MIGHT BE.  WE NEED TO EXECUTE *SOME* ACTION...
      if (winner != -1 &&
	  (winnerPeakedness >= VerbLearn.context().params().minSetFeature() ||
	   f.equals("schema"))) {
	set(f, pfstructs[winner].getDist(f).mode());
	  }
    }
  }

  // Construct a Fstruct from an array of PFstructs as in the above
  // constructor, except that any features in the Fstruct "override"
  // take precedence.
  // THIS IGNORES "OFF" FEATURES IN 'OVERRIDE' ... IS THIS WHAT WE WANT?
  Fstruct(PFstruct[] pfstructs, Fstruct override) {
    this(pfstructs);
    Enumeration fEnum = override.features();
    while (fEnum.hasMoreElements()) {
      String fname = (String)fEnum.nextElement();
      set(fname, override.get(fname));
    }
  }

  // Set the value of a feature.
  // nc: the put is crashing. must be either val is bad
  public void set(String f, String val) { 
    if (VerbLearn.context().params().featureExists(f)) {
      if (f != null && val != null) {	
	//	System.err.println("f is null");
	//{System.err.println("val is null");
	feats.put(f, val);
      }
    }
    else {
      featsOff.put(f, val);
    }
  }

  // hmm -- since either is ok, this causes problems when 
  // we really are just checking about current!
  // Return whether a feature is currently set IN ON
  public boolean isSet(String f) { 
    return feats.containsKey(f);
  }

  // don't think this is necy, but might as well include.
  public boolean isSetIncludingOff(String f) { 
    return feats.containsKey(f) || featsOff.containsKey(f); 
  }

  // Remove a currently set feature.
  public void unset(String f) { 
    feats.remove(f); 
    featsOff.remove(f); 
  }

  // Get the value of a feature.
  public String get(String f) { 
    if (VerbLearn.context().params().featureExists(f)) {
      return (String)feats.get(f); 
    }
    else {
      return (String)featsOff.get(f);
    }
  }

  // Remove features from self which are "explained" by the PFstruct.
  public void explainAway(PFstruct pfstruct) {
    // Loop through all linking features:
    Enumeration fgen = feats.keys();
    while (fgen.hasMoreElements()) {
      String f = (String)fgen.nextElement();
      if (pfstruct.getDist(f).likelihood(get(f)) 
	  > VerbLearn.context().params().minExplain()) {
	unset(f);
      }
    }
  }

  // Returns an Enumeration<String> containing the feature names
  // which are set in this fstruct.  ONLY THOSE WHICH ARE "ON".
  public Enumeration features() { return feats.keys(); }

  // Returns an Enumeration<String> containing the feature names
  // which are set in this fstruct.  ONLY THOSE WHICH ARE "OFF".
  public Enumeration featuresOff() { return featsOff.keys(); }

  // Move features back and forth between feats and featsOff,
  // in accordance with VerbLearn.context().params().
  // Should be called when the latter is changed (eg removed/restored feats).
  public void rearrangeFeatures() {
    Enumeration fEnum = features();
    while (fEnum.hasMoreElements()) {
      String fname = (String)fEnum.nextElement();
      if (!VerbLearn.context().params().featureExists(fname)) {
	Object fval = feats.get(fname);
	feats.remove(fname);
	featsOff.put(fname, fval);
      }
    }
    fEnum = featuresOff();
    while (fEnum.hasMoreElements()) {
      String fname = (String)fEnum.nextElement();
      if (VerbLearn.context().params().featureExists(fname)) {
	Object fval = featsOff.get(fname);
	featsOff.remove(fname);
	feats.put(fname, fval);
      }
    }
  }

  // we only want to print out those that are on?
 public String toString() {
    StringBuffer res = new StringBuffer("{\n");
    Enumeration fEnum = VerbLearn.context().params().featureNames();
    boolean firstTime = true;
    while (fEnum.hasMoreElements()) {
      String fname = (String)fEnum.nextElement();
      if (isSet(fname)) {
	if (firstTime) {
	  firstTime = false;
	}
	else {
	  res.append(" ");
	}
	res.append("  " + fname + "=" + get(fname) + "\n");
      }
    }
    res.append("}\n");
    return res.toString();
  }

  // String version with 4 features per line of text.
  // Include features which are temporarily "off" so we don't lose them.
  // (This is used when saving scenarios.)
  public String toString4IncludingOff() {
    return ts4(true);
  }

  // String version with 4 features per line of text.
  public String toString4() {
    return ts4(false);
  }

  // String version with 4 features per line of text.
  // (Does the work for toString4() and toString4IncludingOff().)
  private String ts4(boolean includeOff) {
    StringBuffer res = new StringBuffer().append("{");
    Enumeration fEnum;
    if (includeOff) {
      fEnum = VerbLearn.context().params().featureNamesIncludingOff();
    }
    else {
      fEnum = VerbLearn.context().params().featureNames();
    }
    boolean firstTime = true;
    int counter = 0;
    while (fEnum.hasMoreElements()) {
      String fname = (String)fEnum.nextElement();
      // nc changed -- correct?
      //      if (isSet(fname)) {   from this to...
      if (isSetIncludingOff(fname)) {   
	if (firstTime) {
	  firstTime = false;
	}
	else {
	  if (counter > 0) res.append(" ");
	  else res.append("\n\t");
	}
	counter = (counter + 1) % 4;  // % is modulus
	res.append(fname + "=" + get(fname));
      }
    }
    res.append("}\n");
    return res.toString();
  }
}

