import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;

///////////////////////////////////////////////////////////////////////
// Labelling of actions Panel
// nc add observer, to check to see if language changes
public class LabelPanel extends Panel implements Observer {

  private Label title;
  private LangChoice langChoice;
 
  private FstructViewer fv;              
  private Button movieButton;
  private Button forwardScene, backScene, fastForwardScene, fastBackScene,
                 addScene, deleteScene, copyScene, selectScenes;
  private Choice sceneChoice;
  
  private TextField[] newLabelField; 
  private Choice[] newLabelChoice;
  private static String choiceMsg= "Choose...";
  private static String revertMsg= "(unchanged)";
  private static String nolabMsg= "(none)";   // nc if *unlabeled*
  private static String emptyMsg= "(empty)";  // for multislot case

  private Panel south, viewer, labelsPanel; 
  private TextField status;
  private Button save;

  // other
  private Enumeration scEnum; // to cycle through for labels
  private int currentScIndex = 0;  // display first scene (what's DISPLAYED)
  private int prevScIndex = 0;
  private Scenario currentSc;
  // to return to prev currentSc when done browsing
  private Vector allScenes; // to keep track in general vector of scenes
                            // (vector: 0-x, bools)
  private Vector selectedScenes; // names of selectedScenes
  private VerbComplex vc;
  private int numSlots;
  private String currentlang;
  Params currentParams; 
  private int skip;    // keep track of how many non-lexemes in label choice
  String generalMsg;
  SelectoMeter selectoMeter;

  // nc changes galore!
  LabelPanel() {

    VerbLearn.context().addObserver(this);
    //    update(VerbLearn.context(), null); // comment out or not?

    // fix this to include possibility of different scenario sets? (not yet)
    allScenes = VerbLearn.context().scenarios().namesVector();
    selectedScenes = new Vector();
    sceneChoice = new Choice();
    
    //System.out.println(""+allScenes.size());
    for (int i=0; i<allScenes.size(); i++){
      // System.out.print(""+i);
       sceneChoice.add((String)(allScenes.elementAt(i)));
    }

    // ? do in update?
    currentlang = VerbLearn.context().langslot();
    currentParams = VerbLearn.context().params();

    numSlots = VerbLearn.context().numSlots();
    skip = (numSlots < 2) ? 1 : 2; // number of labels to skip over...

    setLayout(new BorderLayout());
    
    title = new Label("Action Labelling Window", Label.CENTER);
    title.setFont(new Font("Helvetica", Font.BOLD, 18));
    
    langChoice = new LangChoice();
    Panel northPanel = new Panel();
    northPanel.setLayout(new BorderLayout());
    northPanel.add(title, "North");
    northPanel.add(langChoice, "South");
    add(northPanel, "North");
    
     Panel middlePanel = new Panel();
     middlePanel.setLayout(new FlowLayout());
    

    // SET UP ANIMATION WINDOW HERE...
    // (SHOULD ALTERNATELY SHOW FSTRUCT FOR FAKE SCENARIOS)
    //    movieButton = new Button("Play Movie");
    
     backScene = new Button("<");
     fastBackScene = new Button("<<");
     forwardScene = new Button(">");
     fastForwardScene = new Button(">>");
     deleteScene = new Button("-");
     addScene = new Button("+");
     copyScene = new Button("&");
     selectScenes = new Button("select...");

     Panel buttonPanel = new Panel();
     buttonPanel.setLayout(new FlowLayout());
     buttonPanel.add(fastBackScene);
     buttonPanel.add(backScene);
     buttonPanel.add(forwardScene);
     buttonPanel.add(fastForwardScene);
     buttonPanel.add(addScene);
     buttonPanel.add(copyScene);
     buttonPanel.add(deleteScene);
      
     
     if (currentParams == null) fv = new FstructViewer();  
        else fv = new FstructViewer(currentParams, true);

     viewer = new Panel();
     viewer.setLayout(new BorderLayout());
     viewer.add(fv, "North");
     Panel tempPanel = new Panel();
     tempPanel.setLayout(new BorderLayout());
     tempPanel.add(buttonPanel, "Center");
     Panel tempSubPanel = new Panel();
     tempSubPanel.setLayout(new FlowLayout());
     selectoMeter = new SelectoMeter();
     selectoMeter.setSize(200, 30);
     tempSubPanel.add(sceneChoice);
     tempSubPanel.add(selectoMeter);
     tempPanel.add(tempSubPanel, "North");
     viewer.add(tempPanel, "Center");
     selectoMeter.repaint(); 
     buildLabelsPanel();
     Panel labelsPanelPanel = new Panel();
     labelsPanelPanel.setLayout(new FlowLayout());
     labelsPanelPanel.add(new Label("Enter label:"));
     labelsPanelPanel.add(labelsPanel);
     viewer.add(labelsPanelPanel, "South");
     middlePanel.add(viewer);
     add(middlePanel, "Center");
     south = new Panel();
     status = new TextField("First choose a language.", 60);
     save = new Button("Save Labels");
     generalMsg = "Browse scenarios, or select language and label cycle.";
     status.setText(generalMsg);      
     status.setEditable(false);
     south.add(status);
     south.add(save);
     add(south, "South");
        validate();

    // go ahead and display sc1
    if (allScenes.size()>0) {
          currentSc = VerbLearn.context().scenarios()
                     .get((String)allScenes.elementAt(currentScIndex));
          displayScene(currentScIndex);
    }
    updateButtonStates();
  }


   // build labeling panel, actions, otheractions
  public void buildLabelsPanel() {

    labelsPanel = new Panel();

    numSlots = VerbLearn.context().numSlots();

    // note that when set to 2 for testing something doesn't quite work...
    // by default. no multiple slots yet
    if (numSlots == 0) {
      numSlots = 1; 
    }

    labelsPanel.setLayout(new GridLayout(2,numSlots,5,3));

    newLabelField = new TextField[numSlots];
    newLabelChoice = new Choice[numSlots];

    // place appropriate number of slots and pulldown choice menus
    for (int i = 0; i < numSlots; i++) {
      TextField tempTF = new TextField(nolabMsg, 7);
      labelsPanel.add(tempTF);
      newLabelField[i] = tempTF;
    }

    for (int i = 0; i < numSlots; i++) {
      Choice tempChoice = new Choice();
      tempChoice.add(choiceMsg); 
      //      tempChoice.add(revertMsg); 
      // can't have empty string in one slot
      if (numSlots > 1) {
	tempChoice.add(emptyMsg); 
      }
      labelsPanel.add(tempChoice);
      newLabelChoice[i] = tempChoice; 
    }

    if (VerbLearn.context().langslot() != null) {
      updateLabelLists();
    }
  }


  // Called when labels must be added to pull-down menus for each slot.
  public void updateLabelLists() {
    
    // current labellist *should* have been built up during reading
    // of labels into context.
    //    VerbLearn.context().scenarios().clearLabelList();
    //    VerbLearn.context().scenarios().newLabelList();
        
    // first remove everything
    for (int i = 0; i < numSlots; i++) {
	  newLabelChoice[i].removeAll();
    }

    //now add correct
    for (int i = 0; i < numSlots; i++) {
      newLabelChoice[i].add(choiceMsg); 
      //      tempChoice.add(revertMsg);
      // can't have empty string in one slot
      if (numSlots > 1) {
	newLabelChoice[i].add(emptyMsg); 
      }

      // also add all current labels (not word senses)
      LabelList llist = VerbLearn.context().scenarios().getLabels(i);

      if (llist != null) {
	//	System.err.println("There is a labelList; now on slot " + i);
	Enumeration lEnum = llist.getLabelList();
	while (lEnum.hasMoreElements()) {
	  String l = (String)lEnum.nextElement();
	  newLabelChoice[i].add(l);
	  //	  System.err.println("Adding " + l + "to slot " + i);
	}
      }
    }
  }
  
  // as is, selected scenarios are just unlabelled scenarios 
  public void updateSelectedScenes(){
     selectedScenes.removeAllElements();
     int[] selectedIndices = new int[allScenes.size()];
     for (int i=0; i<allScenes.size(); i++) {
         if (!VerbLearn.context().scenarios()
             .get((String)allScenes.elementAt(i)).isLabelled()){
                  selectedScenes.addElement(allScenes.elementAt(i));
                  selectedIndices[i]=1;                
         }
     }
     int count = 0;
     for (int i=0; i<allScenes.size(); i++){
         if (selectedIndices[i] == 1) count++;
     }
     int[] selectoFeed = new int[count];
     int j=0;
     for (int i=0; i<allScenes.size(); i++ ){
         if (selectedIndices[i] == 1) {
            selectoFeed[j] = i;
            j++;
         }
      }
      selectoMeter.setSelectedIndices(selectoFeed);     
  }
 
  public void updateButtonStates(){
      if (currentScIndex < allScenes.size() -1) forwardScene.setEnabled(true);
          else forwardScene.setEnabled(false);
      if (currentScIndex > 0) backScene.setEnabled(true);
          else backScene.setEnabled(false);
      fastForwardScene.setEnabled(false);
      for (int i = currentScIndex + 1 ; i < allScenes.size(); i++){
          if (selectedScenes.contains(allScenes.elementAt(i))){
             fastForwardScene.setEnabled(true); 
             break;
          }
      }
      fastBackScene.setEnabled(false);
      for (int i = currentScIndex - 1 ; i >=0 ; i--){
          if (selectedScenes.contains(allScenes.elementAt(i))){
             fastBackScene.setEnabled(true); 
             break;
          }
      }
  }


  ////////////////////////////////////////////////////
  // Update from possible context changes

  // note: maybe change other things --e.g. if features added****
  public void update(Observable o, Object junk) {
    //    if (currentlang != null) &&
    //	(currentlang.equals(((Context)o).langslot()))) {
    //      return;
    //    }
    // there's been some important change
    Context c = (Context)o;


    if (numSlots != c.numSlots()) {
      numSlots = c.numSlots();
      skip = (numSlots > 1) ? 1 : 2; // number of labels to skip over...
      // (depends on number of msgs, e.g. choiceMsg)

      // change to CardPanel approach -- switch to appropriate panel!
      // fix this soon...
      // switch (numSlots()) {
      //    case 1:
      //         labelLayout.show(labelsPanel, "one");
      //    case 2:
      //         labelLayout.show(labelsPanel, "two");
      //    case 3:
      //         labelLayout.show(labelsPanel, "three");
      //    }
    }
    // change in language, if not #: label lists should be updated
    if (currentlang != c.langslot()) {
      currentlang = c.langslot();
      updateLabelLists();
    }

    // check to see if viewer might have changed:
    if (currentParams != c.params()) {
      currentParams = c.params();
      fv.reconfigure(currentParams, true, false, false, false);
    }

    // sc lists automatically updates.
    //System.out.println("updating LabelPanel...");
    allScenes = c.scenarios().namesVector(); // generally won't change
    sceneChoice.removeAll();
    for (int i=0; i<allScenes.size(); i++){
       sceneChoice.add((String)allScenes.elementAt(i));
    }
    
    currentScIndex = 0;
    displayScene(currentScIndex);
    updateButtonStates();
    VerbLearn.frame().validate();
    status.setText("Noted changes to context.");
  }

  public void addNewScenario(){
     //System.out.println("would add new");  
     int i = 1;
     while (allScenes.contains("new"+i)) i++;
     String name = "new"+i;
     Scenario newSc = new Scenario(name, new Fstruct(""), new Fstruct(""));
     VerbLearn.context().scenarios().addScenario(name, newSc);
     allScenes.insertElementAt(name, currentScIndex+1);
     sceneChoice.insert(name, currentScIndex+1);
     currentScIndex++;
     displayScene(currentScIndex);
  }

  public void deleteScenario(){
      //System.out.println("would delete");
      String name = currentSc.id();
      VerbLearn.context().scenarios().removeScenario(name);
      allScenes.removeElement(name);
      sceneChoice.remove(name);
      // show next scenario, or previous if on last.
      if (currentScIndex>=allScenes.size()) {
           currentScIndex = allScenes.size() - 1;
      }
      displayScene(currentScIndex);
  }

  public void copyScenario(){
      //System.out.println("would copy");
      int i = 1;
      while (allScenes.contains("copy"+i)) i++;
      String name = "copy"+i;
      Scenario newSc = 
         new Scenario(name, new Fstruct(currentSc.initialLink()),
                            new Fstruct(currentSc.finalLink()));
      VerbLearn.context().scenarios().addScenario(name, newSc);
      allScenes.insertElementAt(name, currentScIndex+1);
      sceneChoice.insert(name, currentScIndex+1);
      currentScIndex++;
      displayScene(currentScIndex);
  }

  
  ////////////////////////////////////////////////////
  // Event-handling
  // to be handled: 
  // browsing, jump to; any label changes; 
  // save.


  public boolean action(Event e, Object junk) {
   if (e.target == save) {
      if (VerbLearn.isApplication()) {
	   saveLabels();
      }
      else {
	   status.setText("Sorry, applet version can't save files.");
      }
      return true;
    }
    else if (e.target == movieButton) {
      status.setText("Play Jack movie of this scene -- but not yet.");
      return true;
    }
    else if (e.target == forwardScene || e.target == fastForwardScene ||
	       e.target == backScene || e.target == fastBackScene ||
             e.target == sceneChoice) {
	prevScIndex = currentScIndex; 
      updateViewer(e);
      return true;
    }
    else if (e.target == copyScene){
       copyScenario();
       return true;
    }
    else if (e.target == addScene){
       addNewScenario();
       return true;
    }
    else if (e.target == deleteScene){
       deleteScenario();
       return true;
    }
    else if (e.target instanceof TextField) {
       // in case non-label text field added later...
       for (int i = 0; i < numSlots; i++) {
            // label 
            if (e.target==newLabelField[i]) break;
            // non-label
            if (i==(numSlots-1)) return false;
       } 
	 handleLabelEntry(e);
       //advances to next scenario if return is hit
       if(currentScIndex<allScenes.size()){
           currentScIndex++;
           displayScene(currentScIndex);
           //select next label
           for (int i = 0; i < numSlots; i++) newLabelField[i].selectAll();
           updateButtonStates();
       }
       return true; 
    }
    else if ((e.target instanceof Choice) && !(e.target==sceneChoice)) {
       handleLabelChoice(e);
       return true;
    }
    else return false;
 }
    


  //////////////////////////////////////////////////
  // change scene displayed in scene viewer
  public void updateViewer(Event e) {
    changeLabel(); // in case typed in label but didn't hit return
    if (e.target == forwardScene) {
      if (currentScIndex < allScenes.size() - 1) {
	currentScIndex++;
      }
    }
    else if (e.target == backScene) {
      if (currentScIndex > 0) {
	currentScIndex--;
      }
    }
    else if (e.target == sceneChoice) {
      currentScIndex = sceneChoice.getSelectedIndex();
    }
    else if (e.target == fastForwardScene) {
      int index = currentScIndex+1;
      while (index < allScenes.size()){
        if (selectedScenes.contains(allScenes.elementAt(index))) break;
        else index++;
      }
      if (index < allScenes.size()) currentScIndex = index;
    }
    else if (e.target == fastBackScene) {
      int index = currentScIndex-1;
      while (index >= 0){
        if (selectedScenes.contains(allScenes.elementAt(index))) break;
        else index--;
      }
      if (index >= 0) currentScIndex = index;
    }
    // send fv the scenario, not the name
    displayScene(currentScIndex);
    updateButtonStates();
    status.setText("Browsing scenarios.");
  }

  public boolean mouseDown(Event e, int x, int y){
     if (e.target == selectoMeter){
       int index = selectoMeter.clickedIndex;
       if (index!=currentScIndex){
           currentScIndex = index;
           displayScene(currentScIndex);
       } 
       return true;
     }
     else return false;
  }
  
  public boolean mouseDrag(Event e, int x, int y){
     if (e.target == selectoMeter){
       int index = selectoMeter.clickedIndex;
       if (index!=currentScIndex){
           currentScIndex = index;
           displayScene(currentScIndex);
       } 
       return true;
     }
     else return false;
  }


  //////////////////////////////////////////////////
  // Main Actions
  //////////////////////////////////////////////////


  //////////////////////////////////////////////////////////////////////
  // Update display of scene, labels
  //////////////////////////////////////////////////////////////////////

  // diplay scene
  private void displayScene(Scenario sc) {
    fv.display(sc);
    sceneChoice.select(sc.id().toString());  
    displayLabels(sc);
    updateSelectoMeter();
  } 
  

  // display scene at given index: currentSc always points to what's viewed
  private void displayScene(int scIndex) {
    if (scIndex < 0 || scIndex >= allScenes.size()){
       System.out.println("scenario index out of bounds");
       return;
    } 
    currentSc = VerbLearn.context().scenarios()
      .get((String)allScenes.elementAt(scIndex));
    displayScene(currentSc);
  }

  public void updateSelectoMeter(){
    updateSelectedScenes();
    selectoMeter.setNumberOfThings(sceneChoice.getItemCount());
    selectoMeter.setCurrentIndex(sceneChoice.getSelectedIndex());
    selectoMeter.repaint();
  }


  // cycle through all slots, displaying (original) values
  private void displayLabels(Scenario sc) {
    if (sc.isLabelled()) {
      //      unlabelButton.enable(); // can always remove them then...
      for (int i = 0; i < numSlots; i++) {
	if (sc.label().slotFilled(i)) {
	  newLabelField[i].setText(sc.label().getWord(i));
	}
	// if slot is empty, but scene is labeled, label with emptyMsg
	else if (numSlots > 1) { // condition not really necessary.
	  newLabelField[i].setText(emptyMsg);
	}
      }
    }
    else {
      for (int i = 0; i < numSlots; i++) {
	newLabelField[i].setText(nolabMsg); // not labeled
      }
    }
  }
  
  ///////////////////////////////////////////////////////////////////////
  // Check to see if textfield is modified
  // (called when field is modified, + enter!)
  public void handleLabelEntry (Event e) {
    for (int i = 0; i < numSlots; i++) {
      if (e.target == newLabelField[i]) {
	String l =  newLabelField[i].getText().trim();
	if (currentSc.isLabelled()) { 
	  if (l.equals(currentSc.label().getWord(i))) {
	    status.setText("No change made to label.");
	  }
	  else {
	    status.setText("");
          changeLabel();
	  }
	}
	else {
	   status.setText("");
         changeLabel();
	}
	newLabelField[i].setText(l); // strip of whitespace in any case	
      }
    }
  }

  ///////////////////////////////////////////////////////////////////////
  // Change labelfield, if there is a change from current;
  // else revert or do nothing.
  // Enable change buttons too.
  // (Choices: emptyMsg/other set to msg
  // if reverting or selected same as before, display old label.

  public void handleLabelChoice (Event e) {
    String newString;
    boolean someChange;
    for (int i = 0; i < numSlots; i++) {
      if (e.target == newLabelChoice[i]) {
         newString = newLabelChoice[i].getSelectedItem();
	   if (!newString.equals(choiceMsg)) {
	     if (currentSc.isLabelled()) {
	       if (newString.equals(currentSc.label().getWord(i))) {
	          status.setText("No change made to label.");
	       }
	       else {
	         newLabelField[i].setText(newString);
               changeLabel();
	       }
	     }
	     // currently null: if not choose or equal, must be diff.
	     else {
	        newLabelField[i].setText(newString);
              changeLabel();
	     }
        }
	  // just is choice message 
	  newLabelChoice[i].select(choiceMsg);
      }
    }
  }  // so any change to any field will simply enableChange


  //////////////////////////////////////////////////    
  // remove label. Will be written to file as nolabMsg "(none)"
  //////////////////////////////////////////////////

  public void unlabel() {
    if (currentSc.isLabelled()) {
       for (int i = 0; i < numSlots; i++) {
            newLabelField[i].setText(nolabMsg);
       }
    }
    else {
       status.setText("No label to remove; no change has been made.");
    }   
  }

  ///////////////////////////////////////////////////////////
  // Change label and update label choice list if necessary.
  // Note: only called if some change has been made.
  ///////////////////////////////////////////////////////////
  public void changeLabel() {

    if (VerbLearn.context().langslot() == null) {
      status.setText("You must choose a language before labelling.\n");
      return;
    }

    VerbComplex vc = getNewLabel(); 
    LabelList labels;
    
    if (vc == null) {
         unlabel(); 
    }
    else if (vc.equals(currentSc.label())) { // no change
      status.setText("No change has been made.");
      return;
    }

    // *CHANGE Scenario Label* and add to pull-down list IF NOT NULL
    VerbLearn.context().scenarios().setLabel(currentSc, vc);
    updateSelectoMeter();

    status.setText("Label change noted.");

    if (vc != null) {
      for (int i = 0; i < numSlots; i++) {
	  labels = VerbLearn.context().scenarios().getLabels(i);
	  if (labels.addLabel(vc.getWord(i))) { // so word should be added
	    newLabelChoice[i].insert(vc.getWord(i), labels.index() + skip);
	    validate();
	  }
      }
    }
  }

  // Retrieve new verb complex from slot(s).
  // Called from changeLabel().
  private VerbComplex getNewLabel() {

    vc = new VerbComplex(numSlots);

    boolean allNull = true;

    // get new labels, stripping input of surrounding whitespace
    for (int i = 0; i < numSlots; i++) {
      String l =  newLabelField[i].getText().trim();
      // fix for multislot
      if (l.equals("") || l.equals(nolabMsg) || l.equals(emptyMsg)){
         l = null;
      } else allNull = false;

      vc.setWord(i, l);
    }
    if (allNull) return null; else return vc;
  }

  public void saveLabels() {
    if (!VerbLearn.isApplication()) return;

    // (no changes)
    if (!VerbLearn.context().scenarios().labelsModified()) 
      return;

    // ask user...
    YesNoDialog checkSave = new YesNoDialog
      (VerbLearn.frame(), "Save labels?", "Do you wish to save current labels?",
       "Yes", "No", null);
    checkSave.show();
    switch (checkSave.answer()) {
    case YesNoDialog.YES:
      boolean success = VerbLearn.context().scenarios().saveLabels();
      if (success) {
	status.setText("Saved labels (if it was needed).");
      }
      else {
	status.setText("Failed to save labels.");
      }
      break;
      //    case YesNoDialog.CANCEL:
      //      cancelQuit = true; break;
      //default:
    case YesNoDialog.NO:
      //      cancelQuit = false; break;
    }
  }
}
