package NTLSystem.verblearn;

import java.io.*;
import java.util.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import NTLSystem.support.*;

///////////////////////////////////////////////////////////////////////
// Top level user interface for VerbLearn system.
// Can be run as either an application or an applet, although you'll
// have to deal with security restrictions if running as an applet.

public class VerbLearn extends Applet {

  // Base directories for the various VerbLearn files.
  // If running as an application, CODE_BASE_DIR and DATA_BASE_DIR
  // are relevant.  If running as an applet, WEB_BASE_DIR (where both
  // the code and the read-only data reside) is relevant.  These
  // "constants" are set during startup.


  public static final int WIDTH = 710;
  public static final int HEIGHT = 550;

  // The current context:
  private static Context context;

  // Top level user interface stuff:
  private static boolean isApplication;
  private static Frame frame;
  private static CheckboxGroup topLevel;
  private static Checkbox quit;  // if application
  private static Checkbox restart;  // if applet
  private static Panel cardPanel;
  private static CardLayout cardLayout;
  private static MainPanel mainPanel;
  private static GenPanel generatePanel; // replaced GeneratePanel
  private static LabelPanel labelPanel;
  private static TrainPanel trainPanel;
  private static LexiconPanel lexiconPanel;
  private static RecognizePanel recognizePanel;
  private static ObeyPanel obeyPanel;
  private static ScrollPane scrollPane; //ok
  
  public static Frame frame() { return frame; }

  public static Context context() { return context; }

  public static boolean isApplication() { return isApplication; }

  // Some panels must be accessible to others and to the Context:

  public static GenPanel generatePanel() { return generatePanel; }
  // repl. GenerateP

  public static LabelPanel labelPanel() { return labelPanel; }

  public static LexiconPanel lexiconPanel() { return lexiconPanel; }

  public static ObeyPanel obeyPanel() { return obeyPanel; }

  public static TextArea trainLog() { return trainPanel.log(); }

  public static void main(String[] args) { 
    isApplication = true;
    if (args.length != 1) {
      System.err.println("You must supply the data directory as an argument.");
      System.exit(1);
    }
    DATA_BASE_DIR = args[0] + "/";
    CODE_BASE_DIR = System.getProperty("CODE_BASE_DIR") + "/";
    System.out.println("DATA:" + DATA_BASE_DIR);
    System.out.println("CODE:" + CODE_BASE_DIR);
    frame = new Frame();
    frame.setTitle("VerbLearn");
    frame.setSize(WIDTH+20, HEIGHT+10);
    VerbLearn vl = new VerbLearn();
    frame.setLayout(new BorderLayout());
    scrollPane = new ScrollPane();
    scrollPane.setSize(WIDTH+12, HEIGHT+2);
    scrollPane.add(vl);
    frame.add(scrollPane, "Center");
    
    vl.init(); 
  }

  public void init() {

    // Early GUI initialization:
    if (frame == null) {
      Container p = getParent();
      // while (!(p instanceof Frame) && (p != null))
      while (!(p instanceof Frame))
	{ p = p.getParent(); }
      frame = (Frame)p;
    }
    setSize(WIDTH, HEIGHT);
    setBackground(new Color(0xddddff));
    setLayout(new GridLayout(1, 1));
    setFont(new Font("Helvetica", Font.PLAIN, 12));
    frame.validate(); 
    frame.show();

    // Figure out where the data will come from...
    if (isApplication) {
      DataSource.setUseFiles(DATA_BASE_DIR);
    }
    else {
      WEB_BASE_DIR = getParameter("WEB_BASE_DIR") + "/";
      System.out.println("WEB_BASE_DIR is " + WEB_BASE_DIR);
      DataSource.setUseWeb(WEB_BASE_DIR);
    }

    // Application initialization:
    context = new Context();
    context.init();

    // Rest of GUI initialization:
 
    Panel bigPanel = new Panel();
    bigPanel.setLayout(new BorderLayout());
    
    Panel northPanel = new Panel();
    northPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

    topLevel = new CheckboxGroup();
    northPanel.add(new Checkbox("Main", topLevel, true));
    northPanel.add(new Checkbox("Generate", topLevel, false));
    northPanel.add(new Checkbox("Label", topLevel, false));
    northPanel.add(new Checkbox("Train", topLevel, false));
    northPanel.add(new Checkbox("Lexicon", topLevel, false));
    northPanel.add(new Checkbox("Recognize", topLevel, false));
    northPanel.add(new Checkbox("Obey", topLevel, false));
    if (isApplication) {
      quit = new Checkbox("Quit");
      northPanel.add(quit);
    }
    else {
      restart = new Checkbox("Restart");
      northPanel.add(restart);
    }
    bigPanel.add("North", northPanel);

    cardPanel = new Panel();
    cardPanel.setSize(WIDTH, HEIGHT-50);
    cardLayout = new CardLayout();
    cardPanel.setLayout(cardLayout);

    mainPanel = new MainPanel();
    // changed from GeneratePanel; needs parent
    generatePanel = new GenPanel(frame); 
    labelPanel = new LabelPanel();
    trainPanel = new TrainPanel();
    lexiconPanel = new LexiconPanel();
    recognizePanel = new RecognizePanel();
    obeyPanel = new ObeyPanel();

    cardPanel.add("Main", mainPanel);
    cardPanel.add("Generate", generatePanel);
    cardPanel.add("Label", labelPanel);
    cardPanel.add("Train", trainPanel);
    cardPanel.add("Lexicon", lexiconPanel);
    cardPanel.add("Recognize", recognizePanel);
    cardPanel.add("Obey", obeyPanel);
    bigPanel.add("Center", cardPanel);
    add(bigPanel);
    cardLayout.show(cardPanel, "Main");

    frame.validate(); frame.repaint();
    frame.show();
  }

  //public void doLayout(){
  //     System.out.println("being layed out");
  //     if (frame !=null && scrollPane != null)
  //         scrollPane.setSize(frame.getSize().width-8, frame.getSize().height-8);
  //    super.layout();
  //}

  public boolean action(Event e, Object arg) {
    if (e.target instanceof Checkbox &&
	((Checkbox)e.target).getCheckboxGroup() == topLevel) {
      if (((Boolean)arg).booleanValue()) {
	cardLayout.show(cardPanel, ((Checkbox)e.target).getLabel());
      }
      return true;
    }
    if (e.target == quit) {
      // saveScenarios(); // nc
      labelPanel().saveLabels();
      System.exit(0);
      return true;
    }
    if (e.target == restart) {
      // saveScenarios();// nc
      labelPanel().saveLabels();
      removeAll();
      init();
      invalidate(); validate();
      return true;
    }
    if (e.id == Event.WINDOW_DESTROY){
      System.exit(0);
    }
    return super.action(e, arg);
  }

  // This is for separate applications which want to use Context-dependent
  // objects.  It basically just establishes VerbLearn.context().
  // Base_dir is either a pathname or URL, as appropriate according to isApp.
  public void outsideInit(boolean isApp, String baseDir) {
    isApplication = isApp;
    // First, figure out where the data will come from...
    if (isApplication) {
      DataSource.setUseFiles(baseDir);
    }
    else {
      DataSource.setUseWeb(baseDir);
    }
    // Application initialization:
    context = new Context();
  }

  // This is moved from GeneratePanel (no longer used).
//   public void saveScenarios() {
//     if (!VerbLearn.isApplication()) return;
//     boolean success = VerbLearn.context().scenarios().save();
//     if (success) {
//       status.setText("Saved scenarios (if it was needed).");
//     }
//     else {
//       status.setText("Failed to save scenarios.");
//     }
//   }
}

