package NTLSystem.verblearn;

import java.io.*;
import java.util.*;
import NTLSystem.*;
import NTLSystem.features.*;
import NTLSystem.support.*;

public class tmpGetData
{
    public static String oldBaseDir = "/u/nchang/SYSTEM-old/verblearn/";

    public static void main(String args[])
    {
	Properties vlProps = null;
	try {
	vlProps =
	    NTLLauncher.loadNTLSystemProperties("verblearn");
	} catch (Exception e) {
	    System.out.println("Exception while loading Properties: " +
			       e.getMessage());
	    System.exit(1);
	} // end try-catch

	// Load the Feature dictionary
	String fdFN =
	    vlProps.getProperty("ntlsys.ntlhome") + "/" +
	    vlProps.getProperty("ntlsys.verblearn.homeDir") +
	    vlProps.getProperty("ntlsys.verblearn.dataDir") +
	    vlProps.getProperty("ntlsys.verblearn.dictDir") +
	    vlProps.getProperty("ntlsys.verblearn.dictFN");

	FeatureDictionary fd = null;
	try {
	    System.out.println("Attmpting to load FeatureDictionary from " +fdFN);
	    fd =
		FeatureDictionary.loadDict( new File( fdFN ) );
	} catch (Exception e) {
	    System.out.println("Exception while attempting to load FeatureDictionary: " +
			       e.getMessage() );
	} // end try-catch
	System.out.println("Loaded FeatureDictionary");
	// Create a new ScenarioSet.
	ScenarioSet sset = new ScenarioSet();
	sset.setName( "Original VerbLearn data" );
	System.out.println("Created ScenarioSet");
	// Add the language "English".
	sset.addLanguage( "English" );
	// Load the Params file from ~dbailey/thesis/sys/param/default and 
	//   create the template fstructs based on the params file.
	System.out.println("About to attempt to load default params");
	try {
	    NestedStreamTokenizer nst =
		new NestedStreamTokenizer(new BufferedReader(new FileReader(oldBaseDir + "param/default" )));
	    Params p = new Params( nst, "default", sset, fd );
	    // Add the params to the scenario set.
	    sset.addParams( p );
	    System.out.println("Added default params");
	} catch (FileNotFoundException fnfe) {
	    System.out.println("FileNotFoundException when attempting to load default params: " + fnfe.getMessage() );
	} // end try-catch
	// Load the Scenarios from ~dbailey/thesis/sys/scenario/specs
	System.out.println("About to read scenarios");
	sset.readScenarios( oldBaseDir + "scenario/specs" );
	// Load the labels from ~dbailey/thesis/sys/lang/English1/labels
	// Label the scenarios
	System.out.println("About to load labels");
	sset.loadLabels( oldBaseDir + "lang/", "English" );
	// Load the restrictions from ~nchang/verblearn/scenario/standardrestrctions
	// Add the restrictions to the ScenarioSet.
	System.out.println("skipping restrictions");
	// Load the datasets from
	// ~dbailey/thesis/sys/dataset/(66-33-shuffled|original-50|second-50)
	String datasetNames[] = { "66-33-shuffled", "original-50", "second-50",
				"asst-8", "empty"};
	for (int i=0; i<datasetNames.length; i++) {
	    try {
		System.out.println("About to load dataset " + datasetNames[i] );
		Dataset dset = new Dataset( datasetNames[i],
					    new NestedStreamTokenizer(new BufferedReader(new FileReader(oldBaseDir + "dataset/" + datasetNames[i]))),
					    sset );
		// Add the dataset to the ScenarioSet.
		System.out.println("About to add dataset " + datasetNames[i]);
		sset.addDataset( dset );
	    } catch (FileNotFoundException fnfe) {
		System.out.println("Couldn't load dataset " + datasetNames[i] +
				   ": " + fnfe.getMessage() );
	    } // end try-catch
	} // end for over dataset names.

	// Save the ScenarioSet to test-old.sset
	System.out.println("About to save ScenarioSet");

	try {
	    String fname =
		vlProps.getProperty("ntlsys.ntlhome") + "/" +
		vlProps.getProperty("ntlsys.verblearn.homeDir") +
		vlProps.getProperty("ntlsys.verblearn.dataDir") +
		vlProps.getProperty("ntlsys.verblearn.ssetDir") +
		"oldData.sset";
	    ObjectOutputStream oos =
		new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(fname)));
	    oos.writeObject( sset );
	    oos.close();
	} catch (IOException ioe) {
	    System.out.println( "Error saving ScenarioSet: " +
				ioe.getMessage() );
	} // end try-catch
	System.out.println("Done saving scenario set");

	System.exit(0);
    }

}



